// T11simReg5.sce
// MULTIVARIATE REGRESSION MODEL AND LS ESTIMATION
// Experiments
// - change parameters of the model
// - change the input signal
// - try to increase the model order to 2
// -------------------------------------------------------------------
exec("ScIntro.sce",-1), mode(0)

// PARAMETERS                                                           // 1
nd=100;                            // length of data                    // 2
a1S=[.8 .2                                                              // 3
    .1 .6];                        // parameters at y(t-1)              // 4
b0S=[1 -.5]';                      // parameters at u(t)                // 5
b1S=[.5 .1]';                      // parameters at u(t-1)              // 6
kS=[-5 3]';                        // constant (model absolute term)    // 7
sS=.1*eye(2,2);                    // noise std                         // 8
ny=max(size(a1S));                 // number of y-variables             // 9
                                                                        // 10
y(1,:)=[1 -1];                     // initial conditions for output     // 11
u=signal(nd,1,.1);                 // input                             // 12
                                                                        // 13
// TIME LOOP                                                            // 14
thS=[b0S a1S b1S kS]';             // matrix of parameters              // 15
V=zeros(7,7);                      // initial statistics                // 16
ka=0;                                                                   // 17
for t=2:nd                                                              // 18
  ps=[u(t) y(t-1,:) u(t-1) 1]';    // regression vector                 // 19
  y(t,:)=thS'*ps+sS*rand(2,1,'n'); // simulation                        // 20
  Y(t,:)=y(t,:);                   // collection of Y                   // 21
  X(t,:)=ps;                       // collection of X                   // 22
end                                                                     // 23
thE=inv(X'*X)*X'*Y;                // parameter estimates               // 24
                                                                        // 25
// RESULTS                                                              // 26
set(gcf(),"position",[700 100 600 500])                                 // 27
subplot(211),plot(1:nd,u),title('Input')                                // 28
subplot(212),plot(1:nd,y),title('Output')                               // 29
disp('The simulated and estimated parameters are',[thS,thE])            // 30
