// T18simState.sce
// SIMULATION WITH REg.MOD. IN A STATE-SPACE FORM
// Experiments
// - extend to the 3-rd order model
//   y(t)=b0.u(t)+a1.y(t-1)+b1.u(t-1)+a2.y(t-2)+b2.u(t-2)+
//        +a3.y(t-3)+b3.u(t-3)+k+e(t)
// -------------------------------------------------------------------
exec("ScIntro.sce",-1)
rand('seed',0), mode(0)

// PARAMETERS OF THE SIMULATION                                         // 1
nd=100;                                           // number of data     // 2
                                                                        // 3
e=rand(1,nd,'n');                                 // regression noise   // 4
u=rand(1,nd,'n');                                 // input              // 5
aS=[.6 .1]; b0S=.8; bS=[.3 .2]; kS=2; cvS=.1;     // model parameterers // 6
                                                                        // 7
// REGRESSION REALIZATION                                               // 8
yr(1)=0; yr(2)=1;                                 // initial conditions // 9
// time loop for simulation using regression model                      // 10
for t=3:nd                                                              // 11
  er=sqrt(cvS)*e(t);                              // regression noise   // 12
  yr(t)=aS*[yr(t-1) yr(t-2)]'+b0S*u(t)+bS*[u(t-1) u(t-2)]'+kS+er;       // 13
                                                  // output             // 14
end                                                                     // 15
                                                                        // 16
// STATE-SPACE REALIZATION                                              // 17
M=[aS(1) bS(1) aS(2) bS(2) kS                     // state parameters   // 18
   0     0     0     0     0                                            // 19
   1     0     0     0     0                                            // 20
   0     1     0     0     0                                            // 21
   0     0     0     0     1];                                          // 22
N=[b0S 1 zeros(1,3)]'                                                   // 23
A=[1 zeros(1,4)];                                                       // 24
B=0;                                                                    // 25
                                                                        // 26
y(1)=0; y(2)=1;                                  // initial conditions  // 27
xt(:,2)=[y(2) u(2) y(1) u(1) 1]';                                       // 28
                                                                        // 29
// time loop of simulation using state-space model                      // 30
for t=3:nd                                                              // 31
  es=[sqrt(cvS)*e(t) zeros(1,4)]';                // state noise        // 32
  xt(:,t)=M*xt(:,t-1)+N*u(t)+es;                  // state update       // 33
  y(t)=A*xt(:,t);                                 // output             // 34
end                                                                     // 35
                                                                        // 36
// RESULTS                                                              // 37
scf(1); plot(1:nd,y,1:nd,yr,'.','markersize',4)                         // 38
legend('state model','regression model');                               // 39
