// T14simExp.sce
// SIMULATION OF EXPONENTIAL MODEL  f(y)=a*exp(-a*y)
// Experiments
// - change parameters of the model
// -------------------------------------------------------------------
exec("ScIntro.sce",-1), mode(0)

// PARAMETRS                                                            // 1
nd=1000;                                                                // 2
thS=[2 5];                              // comp. expectations           // 3
rS=[1 2];                               // comp. variances              // 4
alS=[.4 .6];                            // pointer model parameters     // 5
nc=length(thS);                         // number of components         // 6
                                                                        // 7
// INITIALIZATION                                                       // 8
ni=100;                                 // strength of prior info.      // 9
S=thS*ni; ka=ones(1,nc)*ni;             // initial statistics           // 10
thE=S./ka;                              // initial parametrs            // 11
tht=thE;                                // remember par. estimates      // 12
                                                                        // 13
// ESTIMATION                                                           // 14
for t=1:nd                                                              // 15
  c(t)=sum(cumsum(alS)<rand(1,1,'u'))+1;          // pointer            // 16
  y(t)=thS(c(t))+sqrt(rS(c(t)))*rand(1,1,'n');    // data               // 17
                                                                        // 18
  for j=1:nc                                                            // 19
    q(j)=GaussN(y(t),thE(j),sqrt(rS(j)));         // proximity          // 20
  end                                                                   // 21
                                                                        // 22
  w=q/sum(q);                           // weights                      // 23
  wt(:,t)=w;                                                            // 24
                                                                        // 25
  for j=1:nc                                                            // 26
    S(j)=S(j)+w(j)*y(t);                // statistics                   // 27
    ka(j)=ka(j)+w(j);                                                   // 28
    thE(j)=S(j)/ka(j);                  // point estimates              // 29
  end                                                                   // 30
  tht=[tht; thE];                       // store                        // 31
end                                                                     // 32

// RESULTS
set(scf(1),'position',[600 100 800 300]);
subplot(131),plot(y,'.:')
title 'Data'
subplot(132),plot(wt','.')
title 'Weights'
subplot(133),plot(tht)
title 'Evolution of parametr estimates'
plot([1 nd],[thS(1) thS(1)],':')
plot([1 nd],[thS(2) thS(2)],'g:')
set(gca(),'data_bounds',[0 nd 1 8])
disp('Simulated expectations',thS)
disp('Final estimates of expectations',thE)
