// T31stEst.sce
// STATE ESTIMATION (KALMAN FILTER) 
// Experiments
// - change model parameters M,N,A,B
// - set different system and model covariances rwS,rvS and rwE,rvE
// - try lower stat-estimate covariance Rx
// -------------------------------------------------------------------
exec("ScIntro.sce",-1), mode(0), getd()

nd=200;                            // number of data                       // 1
// SIMULATION                                                              // 2
M=[.8 .1                           // parameters of simulation             // 3
   .3 .6];                                                                 // 4
N=[.5 -.5]';                                                               // 5
A=[.9 -.2];                                                                // 6
B=0;                                                                       // 7
rwS=.1*eye(2,2);                    // noise covariances                   // 8
rvS=.1;                                                                    // 9
x(:,1)=[0 0]';                     // initial state                        // 10
ut=signal(nd,1);                   // input                                // 11
// time loop of simulation                                                 // 12
for t=2:nd                                                                 // 13
  x(:,t)=M*x(:,t-1)+N*ut(t)+rwS*rand(2,1,'n');                             // 14
  y(t)  =A*x(:,t)+B*ut(t)+rvS*rand(1,1,'n');                               // 15
end                                                                        // 16
                                                                           // 17
// ESTIMATION                                                              // 18
// initialization of estimation                                            // 19
rwE=.1*eye(2,2);                    // state noise covariance              // 20
rvE=.1;                             // output noise vovariance             // 21
Rx=1000*eye(2,2);                  // estimated state covariance           // 22
xE(:,1)=zeros(2,1);                // initial state                        // 23
// loop for state estimation                                               // 24
for t=2:nd                                                                 // 25
  [xE(:,t),Rx,yp(t)]=Kalman(xE(:,t-1),y(t),ut(t),M,N,[],A,B,[],rwE,rvE,Rx);// 26
end                                                                        // 27
                                                                           // 28
// RESULTS                                                                 // 29
subplot(311),plot(1:nd,x(1,:),1:nd,xE(1,:))                                // 30
set(gcf(),"position",[700 100 600 500])                                    // 31
title('First state entry')                                                 // 32
legend('state','estimate');                                                // 33
subplot(312),plot(1:nd,x(2,:),1:nd,xE(2,:))                                // 34
title('Second state entry')                                                // 35
legend('state','estimate');                                                // 36
subplot(313),plot(1:nd,y,1:nd,yp')                                         // 37
title('Output')                                                            // 38
legend('output','estimate');                                               // 39
