// T52mixExpNor.sce
// MIXTURE ESTIMATION (descriptive normal)
// - static normal componens
// - multinomial y and v
//   |y1|    |a11 a12 a13||v1|
//   |  | =  |           ||v2| + e
//   |y2|    |a21 a22 a23||v3|
// Experiments
// - change simulated expectations thS
// - change initial expectations thE (through the statistics S, ka)
// -------------------------------------------------------------------
exec("ScIntro.sce",-1), 
getd(), mode(0)

nd=100;                                                                 // 1
// SIMULATION                                                           // 2
thS=list();                                                             // 3
thS(1)=[1 6 10                          // simulated reg. coefficients  // 4
        3 1 2];                                                         // 5
thS(2)=[8 1 5                                                           // 6
        4 6 8];                                                         // 7
thS(3)=[6 9 1                                                           // 8
        5 3 2];                                                         // 9
alS=[.3 .2 .5];                         // parameters of pointer model  // 10
sd=.1;                                  // common and known noise std.  // 11
for t=1:nd                                                              // 12
  jS=sampCat(alS);                      // pointer value generation     // 13                  
  cS(t)=jS;                                                             // 14
  v(:,t)=randn(1,3);                    // expl. variables              // 15
  y(:,t)=thS(jS)*v(:,t)+sd*randn(2,1);  // output value generation      // 16                   
end                                                                     // 17
nc=length(thS);                         // number of componens          // 18
                                                                        // 19
// INITIALIZATION                                                       // 20
V=list(); thE=list();                                                   // 21
ka=[1 1 1];                             // initial counter stat.        // 22        
for j=1:nc                                                              // 23
  thE(j)=thS(j)+5*rand(2,3);            // def. of initial parameters   // 24
  V(j)=eye(5,5);                        // initial inf. matrix          // 25
  V(j)(1:2,3:5)=thE(j);                                                 // 26
  V(j)(3:5,1:2)=thE(j)';                                                // 27
  thE(j)=v2thN(V(j)/ka(j),2);           // initial point estimates      // 28   
end                                                                     // 29
                                                                        // 30
// TIME LOOP                                                            // 31
for t=1:nd                                                              // 32
  // estimation                                                         // 33
  for j=1:nc                                                            // 34
    [nill,qp(j)]=GaussN(y(:,t),thE(j)'*v(:,t),sd*eye(2,2)); //proximity // 35                       
  end                                                                   // 36
  q=exp(qp-max(qp));                                                    // 37
  w=q/sum(q);                           // weights                      // 38   
  wt(:,t)=w;                                                            // 39
  for j=1:nc                                                            // 40
    Ps=[y(:,t)' v(:,t)']';              // extended reg. vector         // 41
    V(j)=V(j)+w(j)*Ps*Ps';              // update of statistics V       // 42
    ka(j)=ka(j)+w(j);                   // update of counter            // 43   
    thE(j)=v2thN(V(j)/ka(j),2);         // point estimates              // 44   
    c(j).th(:,t)=thE(j)(:);             // remember for plot            // 45   
  end                                                                   // 46
                                                                        // 47
  // zero-step prediction                                               // 48         
  ypp=0;                                                                // 49
  for j=1:nc                                                            // 50
    select 1        // type of prediction: 1-point, 2-generated;        // 51
    case 1, ypp=ypp+w(j)*thE(j)'*v(:,t);                                // 52     
    case 2, ypp=ypp+w(j)*(thE(j)'*v(:,t)+sd*randn(2,1));                // 53
    end                                                                 // 54
  end                                                                   // 55
  yp(:,t)=ypp;                            // prediction                 // 56 
end                                                                     // 57
                                                                        // 58
// RESULTS                                                              // 59
set(scf(1),'position',[600 10 1200 400])                                // 60
for j=1:nc                                                              // 61
  subplot(1,3,j)                                                        // 62
  plot(c(j).th')                                                        // 63
  title('Evolution of estimated parametrs: comp. '+string(j))           // 64                
end                                                                     // 65
                                                                        // 66
disp 'Simulated parameter values'                                       // 67
disp(thS(:))                                                            // 68 
disp 'Final estimated parameters'                                       // 69
disp(thE(:))                                                            // 70
                                                                        // 71
set(scf(2),'position',[900 500 600 400])  // plot data and prediction   // 72     
plot(1:nd,y,'ob',1:nd,yp,'xr','markersize',8)                           // 73                     
title 'Data (b) and prediction (r)'                                     // 74  
                                                                        // 75
[nill,cp]=max(wt,'r');                    // accuracy of classification // 76
disp 'Accuracy of classification'                                       // 77
ACC=acc(cS,cp)                                                          // 78
                                                                        // 79
disp 'Relative prediction error'          // accuracy of prediction     // 80
RPE=rpe(y(:),yp(:))                                                     // 81

