// T63mixPreStat.sce
// MIXTURE ESTIMATION AND PREDICTION (predictive, state-space)
// Experiments
// - change: simulated parameters, initial parameters, input signal,
//           type of proximity, length of prediction
// -------------------------------------------------------------------
exec("ScIntro.sce",-1), 
getd(), mode(0)

nd=500;                                                                 // 1  
np=3;                                                                   // 2
// PARAMETERS                                                           // 3
c(1).mS=[-.5 .1                         // component parametrs          // 4
         .2 .7];                                                        // 5
c(2).mS=[.6 .2                                                          // 6
         .1 -.4];                                                       // 7
c(1).nS=[1 -1]';                                                        // 8
c(2).nS=[-1 1]';                                                        // 9
c(1).aS=[.9 1.5];                                                       // 10
c(2).aS=[1.2 .5];                                                       // 11
c(1).bS=10;                                                             // 12
c(2).bS=-1;                                                             // 13
c(1).swS=[.5 .1                         // component covariances        // 14
           0 .2]*.1;                                                    // 15
c(2).swS=[.1 .5                                                         // 16
           0 .2]*.1;                                                    // 17
c(1).svS=.3;                                                            // 18
c(2).svS=.5;                                                            // 19
alS=[.95 .05                            // parameters of pointer model  // 20
     .05 .95];                                                          // 21
nx=size(c(1).mS,1);                                                     // 22
nc=length(c);                           // number of components         // 23
xS(:,1)=zeros(nx,1);                    // initail state                // 24
cS=1;                                   // initial pointer              // 25
u=.5*signal(nd,2,0,4);                  // input                        // 26
                                                                        // 27
// SIMULATION                                                           // 28
for t=2:nd                                                              // 29
  jS=sampCat(alS(:,cS(t-1)));           // pointer value                // 30
  cS(t)=jS;                             // stor pointer value           // 31
  xS(:,t)=c(jS).mS*xS(:,t-1)+c(jS).nS*u(t)+c(jS).swS*randn(2,1);        // 32
  y(t)=c(jS).aS*xS(:,t)+c(jS).bS*u(t)+c(jS).svS*randn();                // 33
end                                                                     // 34
                                                                        // 35
// INITIALIZATION                                                       // 36
ka=[1 1];                               // initial counter              // 37
c(1).x=randn(nx,1);                     // initial state x1             // 38
c(2).x=randn(nx,1);                     // initial state x2             // 39
c(1).R=10*eye(nx,nx);                   // state estimate covariance    // 40
c(2).R=10*eye(nx,nx);                   // state estimate covariance    // 41
c(1).rw=c(1).swS*c(1).swS';             // state model noise            // 42
c(2).rw=c(2).swS*c(2).swS';             // state model noise            // 43
c(1).rv=c(1).svS^2;                     // output model noise           // 44
c(2).rv=c(2).svS^2;                     // output model noise           // 45
                                                                        // 46
// TIME LOOP                                                            // 47
x=zeros(2,nd);                                                          // 48
for t=2:(nd-np)                                                         // 49    
  //proximity                                                           // 50
  for j=1:nc                                                            // 51
    [nill,nill,py,ry] = Kalman(c(j).x(:,t-1),y(t),u(t),..               // 52
                          c(j).mS,c(j).nS,[],c(j).aS,c(j).bS,[],..      // 53
                          c(j).rw,c(j).rv,c(j).R); // prediction        // 54
     select 2            // select type of prox.: 1-quadratic, 2-pdf    // 55
     case 1, lq(j)=-2*log(abs(y(t)-py));     // quadratic proximity     // 56
     case 2, [nill,lq(j)]=GaussN(y(t),py,ry);// pdf proximity           // 57
     end                                                                // 58
  end                                                                   // 59
  // weights                                                            // 60
  q=exp(lq-max(lq));                                                    // 61
  w=q/sum(q);                           // weights                      // 62
  if 0, w=dDel(w); end   // 1=point estimates of weight                 // 63
  wt(:,t)=w;                            // remember weights             // 64
  // estimation and prediction                                          // 65             
  xP(:,t+np)=zeros(nx,1);                                               // 66
  yP(t+np)=0;                                                           // 67
  for j=1:nc                                                            // 68
    // estimation within j-th component                                 // 69
    [c(j).x(:,t),c(j).R,c(j).yp(t)] = Kalman(c(j).x(:,t-1),y(t),u(t),.. // 70
                             c(j).mS,c(j).nS,[],c(j).aS,c(j).bS,[],..   // 71
                             c(j).rw,c(j).rv,c(j).R);                   // 72
     c(j).x(:,t)=limit(c(j).x(:,t),5);  // limiting the state estimate  // 73
    xp=c(j).x(:,t);                                                     // 74
    yp=c(j).yp(t);                                                      // 75
    // prediction within j-th component                                 // 76
    for i=1:np                                                          // 77
      [nill1,nill2,yp,nill4,xp,nill5]=.. // prediction only             // 78
      Kalman(xp,0,u(t+i),c(j).mS,c(j).nS,[],..                          // 79
      c(j).aS,c(j).bS,[],c(j).rw,c(j).rv,c(j).R);                       // 80
    end                                                                 // 81
    // mixture of state and output estimates from comonents             // 82
    xP(:,t+np)=xP(:,t+np)+w(j)*xp;      // resulting state estimate     // 83
    yP(t+np)=yP(t+np)+w(j)*yp;          // resulting prediction         // 84
  end                                                                   // 85
end                                                                     // 86
                                                                        // 87
// RESULTS                                                              // 88
s=(np+1):(nd-np);                                                       // 89
tx=['b';'r';'g'];                                                       // 90
set(scf(1),'position',[600 10 600 800]) // evolution of est. state      // 91
  title 'Simulated and estimated state'                                 // 92
for i=1:nx                                                              // 93
  subplot(nx,1,i)                                                       // 94
  plot(s,xS(i,s),s,xP(i,s))                                             // 95      
  xlabel('x'+string(i))                                                 // 96
  legend('xS','xP');                                                    // 97
end                                                                     // 98
                                                                        // 99
set(scf(2),'position',[1200 10 600 400])// output prediction            // 100
plot(s,y(s),s,yP(s))                                                    // 101
legend('y','yp');                                                       // 102
title 'Output and its prediction'                                       // 103
                                                                        // 104
[nill,cp]=max(wt,'r');                  // accuracy of classification   // 105
disp 'Accuracy of classification'                                       // 106
ACC=acc(cS(s),cp(s))                                                    // 107     
                                                                        // 108
disp 'Relative prediction error of y'   // relative prediction error    // 109
RPE1=rpe(y(s),yP(s))                                                    // 110     
